//-------------------------------------------STM32F4_ADC-------------------------------------------------------

/* Beschreibung:
 * Der ADC wird angesteuert und gewandelte Werte ber die USART versendet.
 */

/*
Hinweis zum Clock fuer 8MHz Quarz, damit er auf 168MHz luft (betrifft das stm32F4Discovery):

in der system_stm32f4xx.c muss eingetragen sein:
PLL_M = 8

in der stm32f4xx.h muss eingetragen sein:
HSE_VALUE = 8000000
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "misc.h"
#include "stm32f4xx_usart.h"
#include "stm32f4xx_adc.h"

#define READ_BUFFER_SIZE 20

volatile char read_buffer[READ_BUFFER_SIZE];
volatile char read_buffer_single;
volatile char string_complete;
volatile int uart_str_count;


void Delay(__IO uint32_t nCount)
{
  while(nCount--)
  {
  }
}


void ADC_INIT_EXAMPLE(void)
{
  ADC_InitTypeDef       ADC_InitStructure;
  ADC_CommonInitTypeDef ADC_CommonInitStructure;
  GPIO_InitTypeDef      GPIO_InitStructure;

  /* Enable ADC3, DMA2 and GPIO clocks ****************************************/
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC3, ENABLE);

  /* Configure ADC3 Channel1 pin as analog input ******************************/
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  /* ADC Common Init **********************************************************/
  ADC_CommonInitStructure.ADC_Mode = ADC_Mode_Independent;
  ADC_CommonInitStructure.ADC_Prescaler = ADC_Prescaler_Div2;
  ADC_CommonInitStructure.ADC_DMAAccessMode = ADC_DMAAccessMode_Disabled;
  ADC_CommonInitStructure.ADC_TwoSamplingDelay = ADC_TwoSamplingDelay_5Cycles;
  ADC_CommonInit(&ADC_CommonInitStructure);

  /* ADC3 Init ****************************************************************/
  ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b;
  ADC_InitStructure.ADC_ScanConvMode = ENABLE;//DISABLE;
  ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
  ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;
  ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_T1_CC1;
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  ADC_InitStructure.ADC_NbrOfConversion = 1;
  ADC_Init(ADC3, &ADC_InitStructure);

  /* ADC3 regular channel7 configuration *************************************/
  ADC_RegularChannelConfig(ADC3, ADC_Channel_1, 1, ADC_SampleTime_28Cycles); // umso hher die SampleTime desto genauer der Wert und desto weniger Crosstalk

  /* Enable ADC3 */
  ADC_Cmd(ADC3, ENABLE);

}

void USART_INIT_EXAMPLE(void)
{
  /* USARTx configured as follow:
        - BaudRate = 19200 baud
        - Word Length = 8 Bits
        - One Stop Bit
        - No parity
        - Hardware flow control disabled (RTS and CTS signals)
        - Receive and transmit enabled
  */
  GPIO_InitTypeDef GPIO_InitStructure;
  USART_InitTypeDef USART_InitStructure;

  /* Enable GPIO clock */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);

  /* Enable UART clock */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);

  /* Connect PXx to USARTx_Tx*/
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource10, GPIO_AF_USART3);

  /* Connect PXx to USARTx_Rx*/
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource11, GPIO_AF_USART3);

  /* Configure USART Tx as alternate function  */
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  /* Configure USART Rx as alternate function  */
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  USART_InitStructure.USART_BaudRate = 19200;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

  /* USART configuration */
  USART_Init(USART3, &USART_InitStructure);

  // Receive Interrupt enable
  USART_ITConfig(USART3,USART_IT_RXNE,ENABLE);

  /* Enable USART */
  USART_Cmd(USART3, ENABLE);
}

void UART_SEND_EXAMPLE(char byte)
{
	  USART_SendData(USART3,byte);
	  /* Loop until the end of transmission */
	  while (USART_GetFlagStatus(USART3, USART_FLAG_TC) == RESET){}
}

void UART_SEND_TEXT_EXAMPLE(char *buffer)
{
	while (*buffer)
	{
		UART_SEND_EXAMPLE(*buffer);
		buffer++;
	}
}


void NVIC_INIT_EXAMPLE(uint16_t IRQ)
{
	NVIC_InitTypeDef NVIC_InitStructure;

	NVIC_InitStructure.NVIC_IRQChannel = IRQ;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE; // IRQ Channel wird enabled
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0; // Die Prioritt --> kleine Zahlen = hohe Prioritt
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_Init(&NVIC_InitStructure);
}


void USART3_IRQHandler(void)
{
	if (USART_GetITStatus(USART3,USART_IT_RXNE) == SET)
	{
		  USART_ClearITPendingBit(USART3,USART_IT_RXNE);

		   // Daten aus dem Puffer lesen
		    read_buffer_single = (USART3->DR & (uint16_t)0x01FF);

		    if ( string_complete==0 )
		         {
		               if (read_buffer_single!='\n' && read_buffer_single!='\r' && uart_str_count<READ_BUFFER_SIZE-1)
		               {
		            	   read_buffer[uart_str_count]=read_buffer_single;
		                   uart_str_count++;
		               }
		               else
		               {
		            	   read_buffer[uart_str_count]='\0';
		                   uart_str_count=0;
		                   string_complete=1;
		               }
		         }
	}
}


int main(void)
{
	uint16_t result;
	uint16_t voltage;
	char text[20];

	SystemInit();

	USART_INIT_EXAMPLE();

	// ist fr die USART Empfangsroutine (wird in diesem Beispiel nicht gebraucht)
	NVIC_INIT_EXAMPLE(USART3_IRQn);

	ADC_INIT_EXAMPLE();

	while(1)
    {
		// Starte eine Conversion
		 ADC_SoftwareStartConv(ADC3);

		// wenn die Conversion fertig ist...
		 if (ADC_GetFlagStatus(ADC3, ADC_FLAG_EOC))
		 {
			// holt sich den Wandelwert
			result = ADC_GetConversionValue(ADC3);

			// rechnet den Wandelwert in Spannung(mV) um
			voltage =(float)2954/(float)4095 * result;

			// in einen String wandeln
			sprintf (text,"%d",voltage);

			// Wert auf der ser. Schnittst. ausgeben
			UART_SEND_TEXT_EXAMPLE(text);
			UART_SEND_TEXT_EXAMPLE("\n\r");
			Delay(10000000);
		 }

    }
}
