//-------------------------------------------STM32F4_DMA_PORT-------------------------------------------------------

/* Beschreibung:
 * Simples Beispiel fr einen DMA-Transfer, welcher eine LED zum Blinken bringt.
 */
/*


Hinweis zum Clock fuer 8MHz Quarz, damit er auf 168MHz luft (betrifft das stm32F4Discovery):

in der system_stm32f4xx.c muss eingetragen sein:
PLL_M = 8

in der stm32f4xx.h muss eingetragen sein:
HSE_VALUE = 8000000
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_tim.h"
#include "misc.h"
#include "stm32f4xx_dma.h"

// PORTA GPIO_Pin_3 wird zwischen 0 und 1 hin und her geschaltet. Sequenz knnte auch lnger sein (Nur ein Beispiel)
uint16_t Port_Values[2] = {0x0,0x8};

// Basisadresse des GPIOA-Port + Offset fr ODR-Register
#define GPIOA_ODR_ADRESS	   0x40020014

void TIM1_Config(void)
{
  TIM_TimeBaseInitTypeDef    TIM_TimeBaseStructure;

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, ENABLE);

  /* Time base configuration */
  TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
  TIM_TimeBaseStructure.TIM_Period = 0xF;
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);

  /* TIM6 enable counter */
  TIM_Cmd(TIM1, ENABLE);
}

void PORT_DMA_TEST_EXAMPLE(void)
{

GPIO_InitTypeDef GPIO_InitStructure;
DMA_InitTypeDef DMA_InitStructure;

  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2 | RCC_AHB1Periph_GPIOA, ENABLE);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

/* TIM1 Configuration ------------------------------------------------------*/
  TIM1_Config();

    DMA_DeInit(DMA2_Stream5); // welcher Stream/Channel verwendet wird ergibt sich aus der DMA2 request mapping-Tabelle im Reference-Datasheet auf S.165
    DMA_InitStructure.DMA_Channel = DMA_Channel_6;
    DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)GPIOA_ODR_ADRESS;
    DMA_InitStructure.DMA_Memory0BaseAddr = (uint32_t)&Port_Values;
    DMA_InitStructure.DMA_DIR = DMA_DIR_MemoryToPeripheral;
    DMA_InitStructure.DMA_BufferSize = 2; // Gibt an wie lang die Sequenz sein soll, die dann auf dem PORT ausgegeben wird.
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
    DMA_InitStructure.DMA_Priority = DMA_Priority_High;
    DMA_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Disable;
    DMA_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_HalfFull;
    DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
    DMA_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;
    DMA_Init(DMA2_Stream5, &DMA_InitStructure);

    DMA_Cmd(DMA2_Stream5, ENABLE);

    TIM_DMACmd(TIM1,TIM_DMA_Update,ENABLE);
}

int main(void)
{

	SystemInit();

	PORT_DMA_TEST_EXAMPLE();

	while(1)
    {

    }
}
