//-------------------------------------------STM32F4_RANDOM_GENERATOR-------------------------------------------------------

/* Beschreibung:
 * Der Zufallsgenerator wird initialisiert und es wird eine Zufallszahl auf der USART ausgegeben.
 */
/*

Hinweis zum Clock fuer 8MHz Quarz, damit er auf 168MHz luft (betrifft das stm32F4Discovery):

in der system_stm32f4xx.c muss eingetragen sein:
PLL_M = 8

in der stm32f4xx.h muss eingetragen sein:
HSE_VALUE = 8000000
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_usart.h"
#include "stm32f4xx_rng.h"


void Delay(__IO uint32_t nCount)
{
  while(nCount--)
  {
  }
}

void RNG_INIT_EXAMPLE(void)
{
	/* Enable RNG clock source */
	  RCC_AHB2PeriphClockCmd(RCC_AHB2Periph_RNG, ENABLE);

	  /* RNG Peripheral enable */
	  RNG_Cmd(ENABLE);
}

uint32_t GET_ZUFALL_EXAMPLE(uint8_t bit)
{
	uint32_t random;
	 /* Wait until one RNG number is ready */
	     while(RNG_GetFlagStatus(RNG_FLAG_DRDY)== RESET)
	      {
	      }

	      /* Begrenzung der Eingabe */
	     if (bit > 32)
	     {
	    	 bit = 32;
	     }

	      random = RNG_GetRandomNumber() >> (32-bit);
	      return random;
}

void USART_INIT_EXAMPLE(void)
{
  /* USARTx configured as follow:
        - BaudRate = 19200 baud
        - Word Length = 8 Bits
        - One Stop Bit
        - No parity
        - Hardware flow control disabled (RTS and CTS signals)
        - Receive and transmit enabled
  */
  GPIO_InitTypeDef GPIO_InitStructure;
  USART_InitTypeDef USART_InitStructure;

  /* Enable GPIO clock */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);

  /* Enable UART clock */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);

  /* Connect PXx to USARTx_Tx*/
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource10, GPIO_AF_USART3);

  /* Connect PXx to USARTx_Rx*/
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource11, GPIO_AF_USART3);

  /* Configure USART Tx as alternate function  */
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  /* Configure USART Rx as alternate function  */
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  USART_InitStructure.USART_BaudRate = 19200;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

  /* USART configuration */
  USART_Init(USART3, &USART_InitStructure);

  /* Enable USART */
  USART_Cmd(USART3, ENABLE);
}

void UART_SEND_EXAMPLE(char byte)
{
	  USART_SendData(USART3,byte);
	  /* Loop until the end of transmission */
	  while (USART_GetFlagStatus(USART3, USART_FLAG_TC) == RESET){}
}

void UART_SEND_TEXT_EXAMPLE(char *buffer)
{
	while (*buffer)
	{
		UART_SEND_EXAMPLE(*buffer);
		buffer++;
	}
}

int main(void)
{
	unsigned int random_number;
	char text[20];

	SystemInit();
	USART_INIT_EXAMPLE();
	RNG_INIT_EXAMPLE();


	while(1)
    {
		random_number = GET_ZUFALL_EXAMPLE(10); // gibt eine x bit Zufallszahl zurck (mgliches Argument ist 0 - 32 bit)
		sprintf (text,"%u",random_number);
		// Wert auf der ser. Schnittst. ausgeben
		UART_SEND_TEXT_EXAMPLE(text);
		UART_SEND_TEXT_EXAMPLE("\n\r");
		Delay(50000000);
    }
}
