//-------------------------------------------STM32F4_USART-------------------------------------------------------

/* Beschreibung:
 * Ganze Strings mit Endezeichen werden interruptgesteuert empfangen und dann zurckgesendet.
 */
/*

Hinweis zum Clock fuer 8MHz Quarz, damit er auf 168MHz luft (betrifft das stm32F4Discovery):

in der system_stm32f4xx.c muss eingetragen sein:
PLL_M = 8

in der stm32f4xx.h muss eingetragen sein:
HSE_VALUE = 8000000
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "misc.h"
#include "stm32f4xx_usart.h"

#define READ_BUFFER_SIZE 20

volatile char read_buffer[READ_BUFFER_SIZE];
volatile char read_buffer_single = 0;
volatile char string_complete = 0;
volatile int uart_str_count = 0;

void Delay(__IO uint32_t nCount)
{
  while(nCount--)
  {
  }
}

void USART_INIT_EXAMPLE(void)
{
  /* USARTx configured as follow:
        - BaudRate = 19200 baud
        - Word Length = 8 Bits
        - One Stop Bit
        - No parity
        - Hardware flow control disabled (RTS and CTS signals)
        - Receive and transmit enabled
  */
  GPIO_InitTypeDef GPIO_InitStructure;
  USART_InitTypeDef USART_InitStructure;

  /* Enable GPIO clock */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);

  /* Enable UART clock */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);

  /* Connect PXx to USARTx_Tx*/
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource10, GPIO_AF_USART3);

  /* Connect PXx to USARTx_Rx*/
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource11, GPIO_AF_USART3);

  /* Configure USART Tx as alternate function  */
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  /* Configure USART Rx as alternate function  */
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  USART_InitStructure.USART_BaudRate = 19200;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

  /* USART configuration */
  USART_Init(USART3, &USART_InitStructure);

  // Receive Interrupt enable
  USART_ITConfig(USART3,USART_IT_RXNE,ENABLE);

  /* Enable USART */
  USART_Cmd(USART3, ENABLE);

}

void UART_SEND_EXAMPLE(char byte)
{
	  USART_SendData(USART3,byte);
	  /* Loop until the end of transmission */
	  while (USART_GetFlagStatus(USART3, USART_FLAG_TC) == RESET){}
}

void UART_SEND_TEXT_EXAMPLE(char *buffer)
{
	while (*buffer)
	{
		UART_SEND_EXAMPLE(*buffer);
		buffer++;
	}
}

void NVIC_INIT_EXAMPLE(uint16_t IRQ)
{
	NVIC_InitTypeDef NVIC_InitStructure;

	NVIC_InitStructure.NVIC_IRQChannel = IRQ;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE; // IRQ Channel wird enabled
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0; // Die Prioritt --> kleine Zahlen = hohe Prioritt
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_Init(&NVIC_InitStructure);
}

void USART3_IRQHandler(void)
{
	if (USART_GetITStatus(USART3,USART_IT_RXNE) == SET)
	{
		  USART_ClearITPendingBit(USART3,USART_IT_RXNE);

		   // Daten aus dem Puffer lesen
		    read_buffer_single = (USART3->DR & (uint16_t)0x01FF);

		    if ( string_complete==0 )
		         {
		    			//Bedingungen fr ein Ende des Strings
		               if (read_buffer_single!='\n' && read_buffer_single!='\r' && uart_str_count<READ_BUFFER_SIZE-1)
		               {
		            	   read_buffer[uart_str_count]=read_buffer_single;
		                   uart_str_count++;
		               }
		               else
		               {
		            	   read_buffer[uart_str_count]='\0';
		                   uart_str_count=0;
		                   string_complete=1;
		               }
		         }
	}
}


int main(void)
{

	// Systemclock auf 168MHz (stm32F4Discovery mit 8MHz Quarz)
	SystemInit();

	// Initialisieren der Schnittstelle
	USART_INIT_EXAMPLE();

	// Interruptcontroller fr RX-Interrupt konfigurieren
	NVIC_INIT_EXAMPLE(USART3_IRQn);


	//nun Text auf der ser. Schnittstelle empfangen und zurcksenden.
while(1)
    {
		if (string_complete)
		{
			// Echo des gesendeten Strings erzeugen
			// Achtung der String muss mit '\n' oder '\r' abgeschlossen werden beim Sender
			UART_SEND_TEXT_EXAMPLE(read_buffer);
			string_complete = 0;
		}
    }
}
